Exception: GoodJob::InterruptError

Inherits:
StandardError
  • Object
show all
Defined in:
lib/good_job/interrupt_error.rb

Overview

Exception raised within a job when it is retried after being interrupted by a SIGKILL or power failure. Include GoodJob::ActiveJobExtensions::InterruptErrors in your job class to enable this behavior, then use retry_on or discard_on to control how interrupted jobs are handled. The error stored in the database for the interrupted execution record itself uses GoodJob::InterruptedError.

Direct Known Subclasses

InterruptedError