Class: GoodJob::Execution
Constant Summary
Constants included
from ErrorEvents
GoodJob::ErrorEvents::DISCARDED, GoodJob::ErrorEvents::HANDLED, GoodJob::ErrorEvents::INTERRUPTED, GoodJob::ErrorEvents::RETRIED, GoodJob::ErrorEvents::RETRY_STOPPED, GoodJob::ErrorEvents::UNHANDLED
Instance Method Summary
collapse
Methods inherited from BaseRecord
bind_value, lease_connection, migrated?, migration_pending_warning!, with_logger_silenced
Instance Method Details
#display_serialized_params ⇒ Object
57
58
59
60
61
|
# File 'app/models/good_job/execution.rb', line 57
def display_serialized_params
serialized_params.merge({
_good_job_execution: attributes.except('serialized_params'),
})
end
|
#filtered_error_backtrace ⇒ Object
63
64
65
|
# File 'app/models/good_job/execution.rb', line 63
def filtered_error_backtrace
Rails.backtrace_cleaner.clean(error_backtrace || [])
end
|
#interrupted_duration ⇒ Object
51
52
53
54
55
|
# File 'app/models/good_job/execution.rb', line 51
def interrupted_duration
return unless error_event == "interrupted" && finished_at && performed_at
finished_at - performed_at
end
|
#last_status_at ⇒ Object
33
34
35
|
# File 'app/models/good_job/execution.rb', line 33
def last_status_at
finished_at || created_at
end
|
#number ⇒ Object
19
20
21
|
# File 'app/models/good_job/execution.rb', line 19
def number
serialized_params.fetch('executions', 0) + 1
end
|
#queue_latency ⇒ Object
Time between when this job was expected to run and when it started running
24
25
26
|
# File 'app/models/good_job/execution.rb', line 24
def queue_latency
created_at - scheduled_at
end
|
#runtime_latency ⇒ Object
Monotonic time between when this job started and finished
29
30
31
|
# File 'app/models/good_job/execution.rb', line 29
def runtime_latency
duration
end
|
#status ⇒ Object
37
38
39
40
41
42
43
44
45
46
47
48
49
|
# File 'app/models/good_job/execution.rb', line 37
def status
if finished_at.present?
if error.present? && job.finished_at.present?
:discarded
elsif error.present?
:retried
else
:succeeded
end
else
:running
end
end
|